﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Sally
{
    public class AudioConfig : MVRScript
    {
        private JSONStorableStringChooser jSpeakerMode;
        private AudioConfiguration audioConfig;

        public override void Init()
        {
            try
            {
                audioConfig = AudioSettings.GetConfiguration();
                // Create choices
                List<string> choices = new List<string>();
                choices.Add("Raw");
                choices.Add("Mono");
                choices.Add("Stereo");
                choices.Add("Quad");
                choices.Add("Surround");
                choices.Add("Mode5point1");
                choices.Add("Mode7point1");
                choices.Add("Prologic");
                // Popup
                jSpeakerMode = new JSONStorableStringChooser("speakerMode", choices, audioConfig.speakerMode.ToString(), "Speaker Mode:");
                RegisterStringChooser(jSpeakerMode);
                UIDynamicPopup popup = CreatePopup(jSpeakerMode);
                jSpeakerMode.setCallbackFunction = SetSpeakerMode;
                LayoutElement layout = popup.GetComponent<LayoutElement>();
                layout.minWidth = layout.preferredWidth = 1060f;
                // Text
                UIDynamicTextField textField = CreateTextField(new JSONStorableString("",
                    "<b><color=#FF0000><size=40>VaM will freeze for a few second when switching.</size></color></b>\n\n" +
                    "<size=40>Info from official Unity code:</size>\n\n" +
                    "<b>Raw</b>\nChannel count is unaffected.\n\n" +
                    "<b>Mono</b>\nChannel count is set to <b>1</b>. The speakers are monaural.\n\n" +
                    "<b>Stereo</b>\nChannel count is set to <b>2</b>. The speakers are stereo. This is the editor default.\n\n" +
                    "<b>Quad</b>\nChannel count is set to <b>4</b>. 4 speaker setup. This includes front left, front right, rear left, rear right.\n\n" +
                    "<b>Surround</b>\nChannel count is set to <b>5</b>. 5 speaker setup. This includes front left, front right, center, rear left, rear right.\n\n" +
                    "<b>Mode5point1</b>\nChannel count is set to <b>6</b>. 5.1 speaker setup. This includes front left, front right, center, rear left, rear right and a subwoofer.\n\n" +
                    "<b>Mode7point1</b>\nChannel count is set to <b>8</b>. 7.1 speaker setup. This includes front left, front right, center, rear left, rear right, side left, side right and a subwoofer.\n\n" +
                    "<b>Prologic</b>\nChannel count is set to <b>2</b>. Stereo output, but data is encoded in a way that is picked up by a Prologic/Prologic2 decoder and split into a 5.1 speaker setup."
                    ));
                layout = textField.GetComponent<LayoutElement>();
                layout.minWidth = layout.preferredWidth = layout.minHeight = layout.preferredHeight = 1060f;
            }
            catch (Exception e) { SuperController.LogError("Exception: " + e); }
        }

        private void SetSpeakerMode(string mode)
        {
            if (String.IsNullOrEmpty(mode))
                return;
            switch (mode)
            {
                case "Raw":
                    audioConfig.speakerMode = AudioSpeakerMode.Raw;
                    break;
                case "Mono":
                    audioConfig.speakerMode = AudioSpeakerMode.Mono;
                    break;
                default:
                case "Stereo":
                    audioConfig.speakerMode = AudioSpeakerMode.Stereo;
                    break;
                case "Quad":
                    audioConfig.speakerMode = AudioSpeakerMode.Quad;
                    break;
                case "Surround":
                    audioConfig.speakerMode = AudioSpeakerMode.Surround;
                    break;
                case "Mode5point1":
                    audioConfig.speakerMode = AudioSpeakerMode.Mode5point1;
                    break;
                case "Mode7point1":
                    audioConfig.speakerMode = AudioSpeakerMode.Mode7point1;
                    break;
                case "Prologic":
                    audioConfig.speakerMode = AudioSpeakerMode.Prologic;
                    break;
            }
            if (!AudioSettings.Reset(audioConfig))
                SuperController.LogMessage("The exact settings specified did not apply. Unity automatically uses the closest match that it supports.");
        }
    }
}